<?php
// Encoding: UTF-8 (without BOM) (auto-detect: °°°°°)
require('_head.inc.php');

if ($msg = UnbSession::GetVar('SuccessMessage', true))
{
	UnbUi::ReportSuccess($msg);
}
?>

<h1>Discussions list</h1>

<?php
try
{
	$searchParameters = array();
	$searchParameters['entrypoint'] = true;
	$searchParameters['sort-createdtime'] = true;
	$searchParameters['sort-reverse'] = true;
	$messageIds = UnbSearch::Query($searchParameters);

	echo '<p>' . count($messageIds) . ' discussions in this list';
	echo ' &nbsp;–&nbsp; <a href="create_message.php" class="buttonlike icon13 create">Create a new message</a>';
	echo '</p>';

	$markup = new UnbMarkup();
	$discussionCount = 0;
	if ($messageIds) foreach ($messageIds as $messageId)
	{
		$discussionCount++;

		$msg = UnbMessage::FindByMessageId($messageId);
		if (!isset($msg)) continue;   // Actually, this should not happen here anymore

		$searchRev = $msg->GetSearchRevision(UnbMessageRevisionContentLevel::Subject, true);
		$latestRev = $msg->GetLatestRevision(UnbMessageRevisionContentLevel::Subject, true);
		if (!isset($searchRev)) continue;   // Actually, this should not happen here anymore
		if (!isset($latestRev)) continue;   // Actually, this should not happen here anymore

		echo '<div class="discussionentry' . ($discussionCount == 1 ? ' first' : '') . '">';
		echo '<div class="subjectline">';
		echo '<big>';
		if ($searchRev->GetContentLevel() >= UnbMessageRevisionContentLevel::Subject)
		{
			if (strlen(trim($searchRev->GetSubject())))
				$text = htmlspecialchars($searchRev->GetSubject());
			else
				$text = '(No subject set)';

			echo '<a href="view_message.php?messageid=' . $msg->GetMessageId() . '">' . $text . '</a>';
		}
		else
		{
			echo '<em>No subject accessible</em>';
		}
		echo '</big>';

		$tags = $searchRev->GetTags(true);
		if ($tags)
		{
			echo ' &nbsp; ';
			$count = 0;
			foreach ($tags as $tag)
			{
				#if ($count++) echo ', ';
				echo '<span class="tag">';
				echo $tag->GetTagName();
				if (($tag->GetFlags() & UnbTagFlag::UserData) && ($tag->GetTagData() != null))
					echo ': ' . htmlspecialchars($tag->GetTagData());
				echo '</span> ';
			}
		}

		UnbUi::MsgRating($msg, 1, 30, 'msg-rating');
		echo '</div>';

		$flags = $msg->GetModerationFlags();

		echo '<div class="infoline">';
		echo 'by <strong>' . $msg->GetOwnerUser()->GetDisplayName() . '</strong>';
		$msgCreatedTime = $msg->GetCreatedTime();
		echo ' &nbsp;–&nbsp; <span title="' . UnbUi::DateTime($msgCreatedTime, true, true) . '">' . UnbUi::RelativeTime($msgCreatedTime) . '</span>';
		if ($searchRev->GetCreatedTime() > $msgCreatedTime)
			echo ' &nbsp;–&nbsp; Altered <span title="' . UnbUi::DateTime($searchRev->GetCreatedTime(), true, true) . '">' . UnbUi::RelativeTime($searchRev->GetCreatedTime()) . '</span>';
		$latestReply = $msg->GetLatestReply();
		if (isset($latestReply))
		{
			$repliesCount = $msg->CountReplies();
			$latestReplyTime = $latestReply->GetCreatedTime();
			echo ' &nbsp;–&nbsp; <strong>' . $repliesCount . '</strong> replies, latest <span title="' . UnbUi::DateTime($latestReplyTime, true, true) . '">' . UnbUi::RelativeTime($latestReplyTime) . '</span>';
		}
		echo '</div>';

		echo '<div class="attachinfoline">';
		if ($searchRev->HasFileData())
		{
			$size = $searchRev->GetFileDataLength() . ' bytes';

			echo 'There is additional data attached:
				<a href="get_data.php?messageid=' . $messageId . '&amp;revision=' . $searchRev->GetRevisionNumber() . '">Download</a> (' . $size . ')<br />';
		}
		echo '</div>';

		echo '<div class="modwarning">';
		if ($flags & UnbMessageModerationFlag::Hidden)
		{
			echo 'This message has been hidden by a moderator. &nbsp; ';
		}
		if ($flags & UnbMessageModerationFlag::Locked)
		{
			echo 'This message has been locked by a moderator. &nbsp; ';
		}
		// If searchRevision != latestRevision, it must be because of a moderation state which is covered below:
		if ($latestRev->GetModerationState() == UnbMessageRevisionModerationState::WaitingForApproval)
		{
			echo 'The latest message revision is waiting for approval. &nbsp; ';
		}
		elseif ($latestRev->GetModerationState() == UnbMessageRevisionModerationState::Locked)
		{
			echo 'The latest message revision has been locked by a moderator. &nbsp; ';
		}
		echo '</div>';
		echo '</div>';
	}
}
catch (UnbSecurityException $ex)
{
	UnbUi::ReportError($ex);
}
?>

<?php
require('_foot.inc.php');
?>