<?php
// UNB Components
// Copyright 2007-2010 by Yves Goergen
// Website: http://unclassified.de/go/unb2
// Licence: GNU General Public License (GPL) Version 3
//
// This file contains the UnbSearch class.

if (!class_exists('UnbDatabase', false)) require_once dirname(__FILE__) . '/database.class.php';
if (!class_exists('UnbMessage', false)) require_once dirname(__FILE__) . '/message.class.php';
if (!class_exists('UnbMessageRevision', false)) require_once dirname(__FILE__) . '/messagerevision.class.php';
if (!class_exists('UnbTag', false)) require_once dirname(__FILE__) . '/tag.class.php';
if (!class_exists('UnbUser', false)) require_once dirname(__FILE__) . '/user.class.php';
if (!class_exists('UnbSession', false)) require_once dirname(__FILE__) . '/session.class.php';

// The search class provides search methods and handles search requests and their results.
//
// It depends on the UnbDatabase class for a database connection. It also uses the classes
// UnbMessage, UnbMessageRevision, UnbTag and UnbUser to handle search queries and results related
// to those item types.
//
class UnbSearch
{
	private function __construct()
	{
	}

	private static function ConstructSqlQuery($searchParameters)
	{
		$sessionUserId = UnbSession::GetSessionUserId();
		$isModerator = UnbSession::IsModerator();
		$isAdministrator = UnbSession::IsAdministrator();

		if (is_int($searchParameters['owner-ids']))
			$searchParameters['owner-ids'] = array($searchParameters['owner-ids']);

		$getTagData = intval($searchParameters['get-tag-data']);

		$parameters = array();
		$sql = '';
		if ($getTagData > 0)
			$sql .= 'SELECT mrtGTD."TagData" AS "TagData", COUNT(*) AS "Count"';
		else
			$sql .= 'SELECT m."MessageId"';

		$sql .= ' FROM "#message" m';

		#if (!$isModerator && !$isAdministrator)
		#	// Require an approved SearchRevision
		#	$sql .= ' LEFT JOIN "#message_revision" mrSearch
		#		ON mrSearch."MessageId" = m."MessageId" AND mrSearch."RevisionNumber" = m."SearchRevision"';
		# -- It's not the task of the search function to try to evaluate ALL moderation restrictions!

		$sortTagId = intval($searchParameters['sort-tagid']);
		if ($sortTagId > 0)
		{
			$sql .= ' JOIN "#message_revision_tag" mrt
				ON mrt."MessageId" = m."MessageId" AND mrt."RevisionNumber" = m."SearchRevision" AND mrt."TagId" = ?';
			$parameters[] = $sortTagId;
		}
#		if ($searchParameters['createdtime-notbefore'] ||
#			$searchParameters['createdtime-notafter'] ||
#			$searchParameters['sort-createdtime'])
#		{
#			$sql .= ' JOIN "#message_revision" mr1st ON
#				mr1st."MessageId" = m."MessageId" AND
#				mr1st."RevisionNumber" =
#					(SELECT MIN(mr2."RevisionNumber")
#					FROM "#message_revision" mr2
#					WHERE mr2."MessageId" = m."MessageId")';
#		}
		if (is_array($searchParameters['tag-value']))
		{
			foreach ($searchParameters['tag-value'] as $tagId => $value)
			{
				if (!is_numeric($tagId)) continue;

				$sql .= ' JOIN "#message_revision_tag" mrTV' . $tagId . '
					ON mrTV' . $tagId . '."MessageId" = m."MessageId" AND
						mrTV' . $tagId . '."RevisionNumber" = m."SearchRevision" AND
						mrTV' . $tagId . '."TagId" = ?';
				$parameters[] = $tagId;
			}
		}

		if ($getTagData > 0)
		{
			$sql .= ' JOIN "#message_revision_tag" mrtGTD ON mrtGTD."MessageId" = m."MessageId" AND mrtGTD."RevisionNumber" = m."SearchRevision" AND mrtGTD."TagId" = ?';
			$parameters[] = $getTagData;
		}

		$sql .= '
			WHERE
				-- Grant all access to administrators (so that they can alter the access key lists)
				(? OR
					-- Message has no read access keylist
					(m."ReadAccessKeylistId" IS NULL
					OR EXISTS
						-- Message\'s read access keylist contains personal or additional user key
						(SELECT 1
						FROM "#message" m2, "#keylist" m2k
						WHERE m2k."KeylistId" = m2."ReadAccessKeylistId" AND
							m2."MessageId" = m."MessageId" AND
							(m2k."UserId" = ? OR
								m2k."UserId" IN
									(SELECT uk."UserId"
									FROM "#user" u, "#keylist" uk
									WHERE uk."KeylistId" = u."AdditionalKeylistId" AND
										u."UserId" = ?))))';
		$parameters[] = $isAdministrator;
		$parameters[] = $sessionUserId;
		$parameters[] = $sessionUserId;

		$dbDriver = UnbDatabase::GetInstance()->GetDriverName();
		if ($dbDriver != 'pgsql')
		{
			$sql .= '
					AND NOT EXISTS
						-- There must not be any tag that denies access
						(SELECT
							EXISTS
								-- Tag\'s read access keylist contains personal or additional user key
								(SELECT 1
								FROM "#keylist" tk
								WHERE tk."KeylistId" = t."ReadAccessKeylistId" AND
									(tk."UserId" = ? OR
										tk."UserId" IN
											(SELECT uk."UserId"
											FROM "#user" u, "#keylist" uk
											WHERE uk."KeylistId" = u."AdditionalKeylistId" AND
												u."UserId" = ?)))
							AS "Allowed"
						FROM "#message_revision_tag" mrt
							JOIN "#tag" t USING ("TagId")
						WHERE mrt."MessageId" = m."MessageId" AND
							mrt."RevisionNumber" = m."SearchRevision" AND
							t."ReadAccessKeylistId" IS NOT NULL   -- Only regard tags with read access keylist (only they can fail the access test)';
			if ($dbDriver == 'sqlite')
				$sql .= '
						GROUP BY 1';   // SQLite requires a GROUP BY clause before HAVING
			$sql .= '
						HAVING NOT "Allowed")';
		}
		else
		{
			// This is for PostgreSQL:
			$sql .= '
					AND NOT EXISTS
						-- There must not be any tag that denies access
						(SELECT "Allowed"
						FROM
							(SELECT
								EXISTS
									-- Tag\'s read access keylist contains personal or additional user key
									(SELECT 1
									FROM "#keylist" tk
									WHERE tk."KeylistId" = t."ReadAccessKeylistId" AND
										(tk."UserId" = ? OR
											tk."UserId" IN
												(SELECT uk."UserId"
												FROM "#user" u, "#keylist" uk
												WHERE uk."KeylistId" = u."AdditionalKeylistId" AND
													u."UserId" = ?)))
								AS "Allowed"
							FROM "#message" m2   -- TODO: remove additional message table reference (m2 = m), see above
								JOIN "#message_revision_tag" mrt
									ON mrt."MessageId" = m2."MessageId" AND mrt."RevisionNumber" = m2."SearchRevision"
								JOIN "#tag" t USING ("TagId")
							WHERE m2."MessageId" = m."MessageId" AND
								-- Only regard tags with read access keylist (only they can fail the access test)
								t."ReadAccessKeylistId" IS NOT NULL) AS foo
						WHERE NOT "Allowed")';
		}
		$parameters[] = $sessionUserId;
		$parameters[] = $sessionUserId;
		$sql .= ')';

		#if (!$isModerator && !$isAdministrator)
		#	// Require an approved SearchRevision
		#	$sql .= '
		#		AND mrSearch."ModerationState" = ' . UnbMessageRevisionModerationState::Approved;
		# -- See corresponding block above.

		if (!$isModerator && !$isAdministrator)
		{
			// Message must not be hidden - message owner is exempted
			$sql .= '
				AND (NOT (m."ModerationFlags" & ' . UnbMessageModerationFlag::Hidden . ') OR
					m."Owner" = ?)';
			$parameters[] = $sessionUserId;
		}

		// The following conditions are not access-related

		if (is_array($searchParameters['message-ids']) && count($searchParameters['message-ids']))
		{
			$messageIds = array_map('intval', $searchParameters['message-ids']);
			$sql .= '
				AND m."MessageId" IN (' . join(', ', $messageIds) . ')';
		}

		if (is_array($searchParameters['owner-ids']) && count($searchParameters['owner-ids']))
		{
			$ownerIds = array_map('intval', $searchParameters['owner-ids']);
			$sql .= '
				AND m."Owner" ' .
				($searchParameters['owner-ids-inverse'] ? 'NOT ' : '') .
				'IN (' . join(', ', $ownerIds) . ')';
		}

		if (is_bool($searchParameters['entrypoint']))
		{
			if ($searchParameters['entrypoint'] === true)
				$sql .= '
					AND m."EntryPoint"';
			elseif ($searchParameters['entrypoint'] === false)
				$sql .= '
					AND NOT m."EntryPoint"';
		}

		if (is_int($searchParameters['primary-reference']))
		{
			if ($searchParameters['primary-reference'] > 0)
			{
				$sql .= '
					AND m."PrimaryReference" = ?';
				$parameters[] = $searchParameters['primary-reference'];
			}
			else
			{
				$sql .= '
					AND m."PrimaryReference" IS NULL';
			}
		}

		if (is_array($searchParameters['tags-set']) && count($searchParameters['tags-set']))
		{
			foreach ($searchParameters['tags-set'] as $tagId)
			{
				$sql .= '
					AND ' . ($tagId < 0 ? 'NOT ' : '') . 'EXISTS
						(SELECT 1
						FROM "#message_revision_tag" mrt
						WHERE mrt."MessageId" = m."MessageId" AND
							mrt."RevisionNumber" = m."SearchRevision" AND
							mrt."TagId" = ?)';
				$parameters[] = abs($tagId);
			}
		}

		if (is_array($searchParameters['tag-value']))
		{
			foreach ($searchParameters['tag-value'] as $tagId => $value)
			{
				if (!is_numeric($tagId)) continue;

				$sql .= ' AND mrTV' . $tagId . '."TagData" ';
				if (is_array($value))
				{
					$sql .= 'BETWEEN ? AND ?';
					$parameters[] = $value[0];
					$parameters[] = $value[1];
				}
				elseif (strpos($value, '*') !== false || strpos($value, '?') !== false)
				{
					#$sql .= 'LIKE ? ESCAPE \'\\\'';
					$sql .= 'LIKE ? ESCAPE ?';
					$parameters[] = UnbDatabase::EscapeLikeWildcards($value);
					$parameters[] = '\\';
				}
				else
				{
					$sql .= '= ?';
					$parameters[] = $value;
				}
			}
		}

		if ($searchParameters['createdtime-notbefore'])
		{
#			$sql .= '
#				AND mr1st."CreatedTime" >= ?';
			$sql .= '
				AND "CreatedTime" >= ?';
			$time = $searchParameters['createdtime-notbefore'];
			if (is_int($time))
				$time = UnbDatabase::ConvertDateTime($time);
			$parameters[] = $time;
		}
		if ($searchParameters['createdtime-notafter'])
		{
#			$sql .= '
#				AND mr1st."CreatedTime" <= ?';
			$sql .= '
				AND "CreatedTime" <= ?';
			$time = $searchParameters['createdtime-notafter'];
			if (is_int($time))
				$time = UnbDatabase::ConvertDateTime($time);
			$parameters[] = $time;
		}

		if ($getTagData > 0)
		{
			$sql .= ' GROUP BY mrtGTD."TagData"';
		}

		if ($sortTagId > 0)
		{
			$sql .= ' ORDER BY mrt."TagData"';
			if ($searchParameters['sort-reverse'])
				$sql .= ' DESC';
		}
		elseif ($searchParameters['sort-createdtime'])
		{
#			$sql .= ' ORDER BY mr1st."CreatedTime"';
			$sql .= ' ORDER BY "CreatedTime"';
			if ($searchParameters['sort-reverse'])
				$sql .= ' DESC';
		}

		if (preg_match('_^\d+(\s*,\s*\d+)?$_', $searchParameters['limit']))
		{
			$sql .= ' LIMIT ' . $searchParameters['limit'];
		}

		return array($sql, $parameters);
	}

	// Processes a search query.
	//
	// query = (string) Query description
	//
	// Throws {{UnbDatabaseException}}
	//
	// Returns (array(int)) Array of message IDs found by the search query
	//
	public static function Query($query = null)
	{
		$searchParameters = array();
		if (is_array($query))
			$searchParameters = $query;
		#$searchParameters['message-ids'] = array(1, 2, 3, 4, 5);
		#$searchParameters['owner-ids'] = array(1, 2);
		#$searchParameters['owner-ids-inverse'] = false;
		#$searchParameters['entrypoint'] = true;
		#$searchParameters['tags-set'] = array(21);
		#$searchParameters['tag-value'][1] = array('2005-01-01', '2009-12-31');
		#$searchParameters['primary-reference'] = 1;
		#$searchParameters['sort-tagid'] = 13;
		#$searchParameters['sort-createdtime'] = false;
		#$searchParameters['sort-reverse'] = false;
		#$searchParameters['limit'] = '';
		#$searchParameters['createdtime-notbefore'] = '2000-01-01';   // or unix timestamp
		#$searchParameters['createdtime-notafter'] = '2020-12-31';   // or unix timestamp

		list($sql, $parameters) = self::ConstructSqlQuery($searchParameters);
		$db = UnbDatabase::GetInstance();
#$t0 = microtime(true);
		$messageIds = $db->QueryArray($sql, $parameters);
#echo round((microtime(true) - $t0) * 1000) . 'ms ';
		if (!isset($messageIds)) return null;
		$messageIds = array_map('intval', $messageIds);

		// All messages returned from this query are accessible by the current session user.
		// To avoid any further checks, cache these message IDs.
		UnbMessage::SetReadAllowedIds($messageIds);

		return $messageIds;
	}

	// Finds all messages that directly or indirectly reference to a given message through their
	// primary references.
	//
	// messageId = (int) Base message ID, the search starts here
	//
	// Returns (array(int => int)) Array of message IDs as keys, with their direct primary reference as value
	//
	public static function GetAllReplies($messageId)
	{
		$out = array();
		self::GetAllReplies_rec(0, $messageId, array(), $out);
		return $out;
	}

	private static function GetAllReplies_rec($primaryRef, $messageId, $seenMessageIds, &$out)
	{
		if (!is_int($messageId))
			throw new InvalidArgumentException('Invalid argument type: $messageId');
		if ($messageId <= 0)
			throw new InvalidArgumentException('Argument out of range: $messageId');

		if ($primaryRef)
			$out[$messageId] = $primaryRef;

		if (in_array($messageId, $seenMessageIds))
		{
			// Reference loop!
			return;
		}

		$seenMessageIds[] = $messageId;

		// Find all messages referencing on this one:
		$searchParameters = array();
		$searchParameters['primary-reference'] = $messageId;
		$messageIds = UnbSearch::Query($searchParameters);

		if ($messageIds)
		{
			foreach ($messageIds as $replyMessageId)
			{
				self::GetAllReplies_rec($messageId, $replyMessageId, $seenMessageIds, $out);
			}
		}
	}

	// Finds all values of a tag with user data.
	//
	// Only messages are considered with a revision accessible to the current session user.
	//
	// query = (string) Query description
	//
	// Throws {{UnbDatabaseException}}
	//
	// Returns (array(string => int)) Array of values (as key) with their ocurrence count
	//
	public static function GetAllTagValues($tagId, $query = null)
	{
		$searchParameters = array();
		if (is_array($query))
			$searchParameters = $query;
		$searchParameters['get-tag-data'] = $tagId;
		$searchParameters['sort-tagid'] = $tagId;

		list($sql, $parameters) = self::ConstructSqlQuery($searchParameters);
		$db = UnbDatabase::GetInstance();
		$records = $db->Query($sql, $parameters);
		if (!isset($records)) return null;
		$a = array();
		foreach ($records as $rec)
			$a[$rec['TagData']] = intval($rec['Count']);
		return $a;
	}

} // class UnbSearch

?>